#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output command log)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2. #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

## Set DISPLAY Parameter
# WFCAMDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep WFCAMDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
 WFCAMDISP=1 
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep WFCAMDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

## Check local command path
PATH=./bin/:$PATH
export PATH

BINPATH=`which fmagnify2`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fmagnify2 found. Check the PATH"
  exit 1
fi 

BINPATH=`which fsubline`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fsubline found. Check the PATH"
  exit 1
fi 

BINPATH=`which fwfcam`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fwfcam found. Check the PATH"
  exit 1
fi 

BINPATH=`which imcopy`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No imcopy found. Check the PATH"
  exit 1
fi 

#Check command arguments
if [ $# != 4 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=maskskyflat.$HEAD.`date +%y%m%d%H%M%S`

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/${COMMANDHEAD}.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

echo "$0 $1 $2 $3 $4"
echo "# $0 $1 $2 $3 $4" > $CLLOG

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi
echo "# Start at $SECSTART" >> $CLLOG

# set filenames
SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits
MSKY=mask${HEAD}_sky${DETECTOR}.fits
MSKYFLAT=mask${HEAD}_skyflat${DETECTOR}.fits
EXPMASK_MSKY=exp_mask${HEAD}_skyflat${DETECTOR}.pl

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ;;
2) DETECTORC=x ;;
3) DETECTORC=y ;;
4) DETECTORC=z ;;
esac

# create local file table
gawk -v path=$DATAPATH -v date=$WFCAMDATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

## Check if image viewer is present
echo ## Check if image viewer is present >> $CLLOG

TMPLOG=$TMPPATH/tmp.log
cat << EOF | cl >$TMPLOG
 display ukirt.fits 1 fi+
 logout
EOF

cat $TMPLOG >> $CLLOG

if [ `grep ERROR $TMPLOG | gawk 'END{print(NR)}'` -gt 0 ]; then
 echo "# No image viewer is present."
 echo "# Set WFCAMDISP=2"
 echo "# No image viewer is present." >> $CLLOG
 echo "# Set WFCAMDISP=2" >> $CLLOG
 WFCAMDISP=2
fi

echo "## Copy data to local disk"
echo "## Copy data to local disk" >> $CLLOG

cat << EOF > $CLCOM
 imdel @ldata.tbl
 imdel @tdata.tbl
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s\n",$1,$2,dark)}'  >> $CLLOGCOM
paste data.tbl ldata.tbl | gawk -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s\n",$1,$2,dark)}' | csh 1>> $CLLOG 

echo "## Fix Header Information"
echo "## Fix Header Information" >> $CLLOG
gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM

# 1st imstat
cat << EOF >> $CLCOM
 del $TMPPATH/tmp.dat
 imstati @ldata-1.tbl format- > $TMPPATH/tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

# 2nd imstat
gawk -v tmppath=$TMPPATH 'BEGIN{printf("del %s/tmp.dat\n",tmppath)}{printf("imstati %s upper=%f lower=%f format- >> %s/tmp.dat\n",$1,$3+3*$4,$3-3*$4,tmppath)}' $TMPPATH/tmp.dat > $CLCOM
echo logout >>$CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Uniform background level"
echo "## Uniform Background level" >> $CLLOG
gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]/s)}' $TMPPATH/tmp.dat > $TMPPATH/tmp2.dat

cat << EOF > $CLCOM
 imarith @ldata.tbl / @${TMPPATH}/tmp2.dat @ldata.tbl
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Register mask files to header"
echo "## Register mask files to header" >> $CLLOG
\rm $CLCOM
ndith=`gawk 'END{print(NR)}' mask${DETECTOR}.tbl`
for dnumber in `gawk '{print(NR)}' ldata.tbl`;do
 dnumber2=`echo $dnumber | gawk -v ndith=$ndith '{print(($1-1)%ndith+1)}'`
 echo $dnumber $dnumber2  > $TMPPATH/tmpnumlist
 MASKFILE=`sed -n ${dnumber2}p mask${DETECTOR}.tbl`
 LFILE=`sed -n ${dnumber}p ldata.tbl`
 echo hedit $LFILE bpm $MASKFILE add+ ver- >> $CLCOM
done
echo logout >> $CLCOM

runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Combine and create a sky frame"
echo "## Combine and create a sky frame" >> $CLLOG
cat << EOF > $CLCOM
 del $MSKY
 imcombine @ldata.tbl $MSKY expmasks="$EXPMASK_MSKY" combine=median reject=sigclip outtype=real masktype=goodvalue maskvalue=0

 del $TMPPATH/tmp.dat
 imstati $MSKY format- > $TMPPATH/tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

if [ $WFCAMDISP -eq 0 ]; then
 cat <<EOF
  disp $MSKY 1 fi+
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 
fi

IMSTATUPPER=`gawk '{print($3+$4*3)}' $TMPPATH/tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' $TMPPATH/tmp.dat`

echo "## Normalize to create a skyflat"
echo "## Normalize to create a skyflat" >> $CLLOG
cat <<EOF > $CLCOM
del $TMPPATH/tmp.dat
imstati $MSKY format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > $TMPPATH/tmp.dat
imdel $MSKYFLAT
imarith $MSKY / @${TMPPATH}/tmp.dat $MSKYFLAT
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Fix bad pixels"
echo "## Fix bad pixels" >> $CLLOG

cat << EOF > $CLCOM
 imdel $TMPPATH/tmp.fits
 imcopy $MSKYFLAT $TMPPATH/tmp.fits
 imreplace $TMPPATH/tmp.fits -1 upper=0.01 
 imreplace $TMPPATH/tmp.fits 0 lower=0
 imreplace $TMPPATH/tmp.fits 1 upper=-0.5
 fixpix $MSKYFLAT $TMPPATH/tmp.fits
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "# Sky frame saved as $MSKY"
echo "# Skyflat frame saved as $MSKYFLAT"
echo "# Skyflat exposure map saved as $EXPMASK_MSKY"

echo "# Sky frame saved as $MSKY" >> $CLLOG
echo "# Skyflat frame saved as $MSKYFLAT" >> $CLLOG
echo "# Skyflat exposure map saved as $EXPMASK_MSKY" >> $CLLOG


#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0

