#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output commandlist)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2 #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

if [ `env | grep MASKRADIUS |gawk 'END{print(NR)}'` -eq 0 ]; then
 MASKRADIUS=6 
fi

MASKRADIUS=`echo $MASKRADIUS| gawk '{print(int($1/6*10))}'`
MASKRADIUS2=`echo $MASKRADIUS| gawk '{print(int($1*2))}'`

## Set DISPLAY Parameter
# WFCAMDISP=0; Display all images
#            1; Display minimum images
#            2; Display No image
if [ `env | grep WFCAMDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
 WFCAMDISP=1 
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep WFCAMDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

## Check local command path
PATH=./bin/:$PATH
export PATH

BINPATH=`which fmagnify2`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fmagnify2 found. Check the PATH"
  exit 1
fi 

BINPATH=`which fsubline`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fsubline found. Check the PATH"
  exit 1
fi 

BINPATH=`which fwfcam`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No fwfcam found. Check the PATH"
  exit 1
fi 

BINPATH=`which imcopy`
PATHSTATUS=`echo $BINPATH | gawk '{print(NF)}'`
if [ $PATHSTATUS != 1 ]; then
  echo "Error : No imcopy found. Check the PATH"
  exit 1
fi 

#Check command arguments
if [ $# != 4 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=objsub.$HEAD.`date +%y%m%d%H%M%S`

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/objsub.$HEAD.`date +%y%m%d%H%M%S`.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "$0 $1 $2 $3 $4"
echo "# $0 $1 $2 $3 $4" > $CLLOG

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMDATE=$WFCAMDATE >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi
echo "# Start at $SECSTART" >> $CLLOG

# set filenames
SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
BADPIX=badpix${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
esac

# create local file table
gawk -v path=$DATAPATH -v date=$WFCAMDATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

## Check if image viewer is present
echo ## Check if image viewer is present >> $CLLOG

TMPLOG=$TMPPATH/tmp.log
cat << EOF | cl >$TMPLOG
 display ukirt.fits 1 fi+
 logout
EOF

cat $TMPLOG >> $CLLOG

if [ `grep ERROR $TMPLOG | gawk 'END{print(NR)}'` -gt 0 ]; then
 echo "# No image viewer is present."
 echo "# Set WFCAMDISP=2"
 echo "# No image viewer is present." >> $CLLOG
 echo "# Set WFCAMDISP=2" >> $CLLOG
 WFCAMDISP=2
fi

echo "## Copy data to local disk with dark subtraction and flat-fielding"
echo "## Copy data to local disk with dark subtraction and flat-fielding" >> $CLLOG

cat << EOF > $CLCOM
 imdel @ldata.tbl
 imdel @tdata.tbl >> $CLCOM
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v flat=$SKYFLAT -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s %s\n",$1,$2,dark,flat)}' | csh 1>> $CLLOG

echo "## Fix Header Information"
echo "## Fix Header Information" >> $CLLOG
gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM

# 1st imstat
cat << EOF >> $CLCOM
 del ${TMPPATH}/tmp.dat
 imstati @ldata-1.tbl format- > ${TMPPATH}/tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

# 2nd imstat
gawk -v tmppath=$TMPPATH 'BEGIN{printf("del %s/tmp.dat\n",tmppath)}{printf("imstati %s upper=%f lower=%f format- >> %s/tmp.dat\n",$1,$3+3*$4,$3-3*$4,tmppath)}' $TMPPATH/tmp.dat > $CLCOM
echo logout >>$CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "## Subtract Background offsets"
echo "## Subtract Background offsets" >> $CLLOG

for dnumber in `gawk  '{print(NR)}' ldata.tbl`;do
 masknumber=`gawk 'END{print(NR)}' offset.tbl`
  # if No of mask smaller than No of frames, take mod.
 dnumber2=`echo $dnumber $masknumber | gawk '{printf("%03d",(($1-1)%$2)+1)}'`
 FRAME=`sed -n ${dnumber}p ldata.tbl` 
 MASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.pl
 TEXTMASKFILE1=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo
 TEXTMASKFILE2=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo2
 
 echo "# Processing $FRAME using $TEXTMASKFILE1"
 cat << EOF > $CLCOM
  imdel tmpmod1.fits
  imcopy $FRAME tmpmod1.fits
  fixpix tmpmod1.fits $BADPIX 
  imedit tmpmod1.fits "" cursor=$TEXTMASKFILE1 radius=$MASKRADIUS buffer=5 width=5 default="b" display-
  imedit tmpmod1.fits "" cursor=$TEXTMASKFILE2 radius=$MASKRADIUS2 buffer=15 width=10 default="b" display-

  imdel mod1.fits,mod2.fits,mod3.fits,mod4.fits
  imcopy tmpmod1.fits[1:1024,1:1024] mod1.fits
  !./bin/fsubline mod1.fits mod1.fits x 5 2 mean
  imarith $FRAME[1:1024,1:1024] - mod1.fits mod2.fits
  imarith tmpmod1.fits[1:1024,1:1024] - mod1.fits mod3.fits
  imsurfit mod3.fits mod4.fits 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith mod2.fits - mod4.fits mod2.fits
  imcopy mod2.fits $FRAME[1:1024,1:1024]

  imdel mod1.fits,mod2.fits,mod3.fits,mod4.fits
  imcopy tmpmod1.fits[1:1024,1025:2048] mod1.fits
  !./bin/fsubline mod1.fits mod1.fits y 5 2 mean
  imarith $FRAME[1:1024,1025:2048] - mod1.fits mod2.fits
  imarith tmpmod1.fits[1:1024,1025:2048] - mod1.fits mod3.fits
  imsurfit mod3.fits mod4.fits 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith mod2.fits - mod4.fits mod2.fits
  imcopy mod2.fits $FRAME[1:1024,1025:2048]
 
  imdel mod1.fits,mod2.fits,mod3.fits,mod4.fits
  imcopy tmpmod1.fits[1025:2048,1:1024] mod1.fits
  !./bin/fsubline mod1.fits mod1.fits y 5 2 mean
  imarith $FRAME[1025:2048,1:1024] - mod1.fits mod2.fits
  imarith tmpmod1.fits[1025:2048,1:1024] - mod1.fits mod3.fits
  imsurfit mod3.fits mod4.fits 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith mod2.fits - mod4.fits mod2.fits
  imcopy mod2.fits $FRAME[1025:2048,1:1024]

  imdel mod1.fits,mod2.fits,mod3.fits,mod4.fits
  imcopy tmpmod1.fits[1025:2048,1025:2048] mod1.fits
  !./bin/fsubline mod1.fits mod1.fits x 5 2 mean
  imarith $FRAME[1025:2048,1025:2048] - mod1.fits mod2.fits
  imarith tmpmod1.fits[1025:2048,1025:2048] - mod1.fits mod3.fits
  imsurfit mod3.fits mod4.fits 4 4 xmedian=5 ymedian=5 type_ou=fit border=10 cross_t=yes
  imarith mod2.fits - mod4.fits mod2.fits
  imcopy mod2.fits $FRAME[1025:2048,1025:2048]

  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 
done

#echo "## Uniform background level"
#echo "## Uniform Background level" 
#gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]-s)}' ${TMPPATH}/tmp.dat > ${TMPPATH}/tmp2.dat
#
#cat << EOF > $CLCOM
# imarith @ldata.tbl - @${TMPPATH}/tmp2.dat @ldata.tbl
# logout
#EOF
#runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "## Divide pixels into 2x2"
echo "## Divide pixels into 2x2" >> $CLLOG
if [ -f $BADPIX ]; then
cat <<EOF > $CLCOM
imdel ${TMPPATH}/tmpbadpix.pl
#magnify $BADPIX ${TMPPATH}/tmpbadpix.pl 2 2 fluxcon-
!fmagnify2 $BADPIX ${TMPPATH}/tmpbadpix.fits
imcopy ${TMPPATH}/tmpbadpix.fits ${TMPPATH}/tmpbadpix.pl
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM
fi

# divide data into 2x2 and add badpixel mask data to header
cat <<EOF > $CLCOM
imdel @tdata.tbl
#magnify @ldata.tbl @tdata.tbl 2 2 interpo=nearest
EOF
for dnumber in `gawk  '{print(NR)}' tdata..tbl`;do
 INFRAME=`sed -n ${dnumber}p ldata.tbl` 
 OUTFRAME=`sed -n ${dnumber}p tdata.tbl` 
 echo \!fmagnify2 $INFRAME $OUTFRAME >> $CLCOM
 if [ -f $BADPIX ]; then
   echo hedit $OUTFRAME bpm ${TMPPATH}/tmpbadpix.pl add+ ver- >> $CLCOM
 fi
done
echo logout >> $CLCOM
runiraf $CLCOM $CLLOG $CLLOGCOM


echo "## Shift and Add all frames"
echo "## Shift and Add all frames" >> $CLLOG

cat <<EOF > $CLCOM
imdel $FINAL
 imcombine @tdata.tbl $FINAL combine=average reject=sigclip offsets=offset.tbl masktype=goodvalue maskvalue=0
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

if [ $WFCAMDISP -lt 2 ]; then
 cat <<EOF > $CLCOM
  disp $FINAL 2 fi+
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM
fi


echo "# Final frame saved as $FINAL"

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0




